"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var ReactDOM = require("react-dom");
var prefix_1 = require("@devexperts/utils/dist/dom/prefix");
var classnames = require("classnames");
var pure_1 = require("../../utils/pure");
var BoundsUpdateDetector_1 = require("../BoundsUpdateDetector/BoundsUpdateDetector");
var throttle_1 = require("@devexperts/utils/dist/function/throttle");
var withTheme_1 = require("../../utils/withTheme");
var EventListener_1 = require("../EventListener/EventListener");
var RootClose_1 = require("../RootClose/RootClose");
var with_defaults_1 = require("../../utils/with-defaults");
var react_dom_1 = require("react-dom");
exports.POPOVER = Symbol('Popover');
var PopoverPlacement;
(function (PopoverPlacement) {
    PopoverPlacement["Top"] = "Top";
    PopoverPlacement["Bottom"] = "Bottom";
    PopoverPlacement["Left"] = "Left";
    PopoverPlacement["Right"] = "Right";
})(PopoverPlacement = exports.PopoverPlacement || (exports.PopoverPlacement = {}));
var PopoverAlign;
(function (PopoverAlign) {
    PopoverAlign["Top"] = "Top";
    PopoverAlign["Bottom"] = "Bottom";
    PopoverAlign["Left"] = "Left";
    PopoverAlign["Right"] = "Right";
    PopoverAlign["Middle"] = "Middle";
    PopoverAlign["Center"] = "Center";
})(PopoverAlign = exports.PopoverAlign || (exports.PopoverAlign = {}));
var RawPopover = /** @class */ (function (_super) {
    tslib_1.__extends(RawPopover, _super);
    function RawPopover(props) {
        var _this = _super.call(this, props) || this;
        _this.state = {};
        _this._needsUpdate = false;
        _this.updatePosition = function () {
            var anchorRect = _this.getAnchorProperties();
            if (!anchorRect) {
                return;
            }
            var _a = _this.props, placement = _a.placement, align = _a.align, hasArrow = _a.hasArrow;
            var arrowOffset;
            var finalPlacement;
            var finalAlign;
            var topResult = movePopoverVertically(placement, align, anchorRect.top, anchorRect.bottom, _this._popoverSize.height, anchorRect.parentHeight, true);
            var leftResult = movePopoverHorizontally(placement, align, anchorRect.left, anchorRect.right, _this._popoverSize.width, anchorRect.parentWidth, true);
            //additional
            if (placement === PopoverPlacement.Top || placement === PopoverPlacement.Bottom) {
                finalPlacement = topResult.placement;
                finalAlign = leftResult.align;
                if (hasArrow) {
                    //horizontal arrow offset
                    arrowOffset = Math.round((anchorRect.right - anchorRect.left) / 2);
                }
            }
            else if (placement === PopoverPlacement.Left || placement === PopoverPlacement.Right) {
                finalPlacement = leftResult.placement;
                finalAlign = topResult.align;
                if (hasArrow) {
                    //vertical arrow offset
                    arrowOffset = Math.round((anchorRect.bottom - anchorRect.top) / 2);
                }
            }
            _this.setState({
                top: Math.round(topResult.top) + window.pageYOffset,
                left: Math.round(leftResult.left) + window.pageXOffset,
                finalPlacement: finalPlacement,
                finalAlign: finalAlign,
                arrowOffset: arrowOffset,
            });
        };
        _this.onSizeUpdate = function (newSize) {
            _this._popoverSize = newSize;
            _this.updatePosition();
        };
        _this.onResize = function () {
            _this.handleResize();
        };
        _this.onScroll = function (event) {
            if (_this._anchor && _this._popover && event.target) {
                var popoverNode = react_dom_1.findDOMNode(_this._popover);
                var scrolled = event.target;
                if (popoverNode && scrolled.contains(_this._anchor)) {
                    _this.handleScroll();
                }
            }
        };
        _this.rootElement = document.createElement('div');
        var container = props.container || document.body;
        container.appendChild(_this.rootElement);
        _this.handleResize = throttle_1.default(_this.handleResize, 100);
        _this.handleScroll = throttle_1.default(_this.handleScroll, 100);
        return _this;
    }
    RawPopover.prototype.componentDidMount = function () {
        var _this = this;
        if (this.props.isOpened) {
            if (this.props.anchor) {
                var anchorDOMNode = react_dom_1.findDOMNode(this.props.anchor);
                if (!anchorDOMNode || anchorDOMNode instanceof Text) {
                    return;
                }
                this._anchor = anchorDOMNode;
            }
            this._popoverSize = this.getPopoverSize();
            this.updatePosition();
        }
        if (window.matchMedia) {
            this.mediaQueryList = window.matchMedia('print');
            this.mediaQueryList.addListener(function () { return _this.updatePosition(); });
        }
        window['onbeforeprint'] = this.updatePosition;
        window['onafterprint'] = this.updatePosition;
    };
    RawPopover.prototype.componentWillUnmount = function () {
        var container = this.props.container || document.body;
        container.removeChild(this.rootElement);
        this.mediaQueryList && this.mediaQueryList.removeListener(this.updatePosition);
        window['onbeforeprint'] = null;
        window['onafterprint'] = null;
    };
    RawPopover.prototype.componentWillReceiveProps = function (nextProps) {
        if (nextProps.isOpened && nextProps.anchor) {
            this._needsUpdate = true;
            var anchorDOMNode = react_dom_1.findDOMNode(nextProps.anchor);
            if (!anchorDOMNode || anchorDOMNode instanceof Text) {
                return;
            }
            this._anchor = anchorDOMNode;
        }
    };
    RawPopover.prototype.componentDidUpdate = function (prevProps) {
        //@PURE check is passed here - update anyway
        if (this._needsUpdate) {
            /** set in {@link componentWillReceiveProps} */
            this._needsUpdate = false;
            this._popoverSize = this.getPopoverSize();
            this.updatePosition();
        }
    };
    RawPopover.prototype.render = function () {
        var _this = this;
        var _a;
        var _b = this.props, closeOnClickAway = _b.closeOnClickAway, theme = _b.theme, hasArrow = _b.hasArrow, onMouseDown = _b.onMouseDown, isOpened = _b.isOpened, onRequestClose = _b.onRequestClose, anchor = _b.anchor;
        if (!isOpened || !anchor) {
            return null;
        }
        var _c = this.state, top = _c.top, left = _c.left, arrowOffset = _c.arrowOffset, finalPlacement = _c.finalPlacement, finalAlign = _c.finalAlign;
        var isMeasured = !!finalAlign && !!finalPlacement;
        var style;
        var popoverClassName = classnames(theme.container);
        if (isMeasured && finalPlacement) {
            style = prefix_1.default({
                transform: "translate(" + (left || 0) + "px, " + (top || 0) + "px)",
            });
            popoverClassName = classnames(popoverClassName, (_a = {},
                _a[theme.container_hasArrow] = hasArrow,
                _a), [getPlacementModifier(finalPlacement)].map(function (mod) { return theme["container_" + mod]; }));
        }
        style = tslib_1.__assign({}, style, this.props.style);
        var child = (React.createElement(BoundsUpdateDetector_1.BoundsUpdateDetector, { onUpdate: this.onSizeUpdate },
            React.createElement("div", { ref: function (el) { return (_this._popover = el); }, style: style, onMouseDown: onMouseDown, onClick: stopPropagation, className: popoverClassName },
                React.createElement("div", { className: theme.content },
                    isMeasured &&
                        finalPlacement &&
                        finalAlign &&
                        hasArrow && (React.createElement("div", { className: theme.arrow, style: getArrowStyle(finalPlacement, finalAlign, arrowOffset) })),
                    this.props.children))));
        if (closeOnClickAway) {
            child = React.createElement(RootClose_1.RootClose, { onRootClose: onRequestClose }, child);
        }
        var target = typeof window !== 'undefined' ? window : 'window';
        return (React.createElement(EventListener_1.EventListener, { onResize: this.onResize, onScrollCapture: this.onScroll, target: target }, ReactDOM.createPortal(child, this.rootElement)));
    };
    RawPopover.prototype.getPopoverSize = function () {
        var popover = react_dom_1.findDOMNode(this._popover);
        if (popover instanceof HTMLElement) {
            return {
                height: popover.offsetHeight,
                width: popover.offsetWidth,
            };
        }
        return {
            height: 0,
            width: 0,
        };
    };
    RawPopover.prototype.getAnchorProperties = function () {
        var container = this.props.container;
        var anchor = this._anchor;
        if (container && anchor) {
            var containerRect = container.getBoundingClientRect();
            var anchorRect = anchor.getBoundingClientRect();
            return {
                left: anchorRect.left - containerRect.left,
                right: anchorRect.right - containerRect.left,
                top: anchorRect.top - containerRect.top,
                bottom: anchorRect.bottom - containerRect.top,
                width: anchorRect.width,
                parentWidth: containerRect.width,
                parentHeight: containerRect.height,
            };
        }
        else if (anchor) {
            var anchorRect = anchor.getBoundingClientRect();
            return {
                left: anchorRect.left,
                right: anchorRect.right,
                top: anchorRect.top,
                bottom: anchorRect.bottom,
                width: anchorRect.width,
                parentWidth: window.innerWidth,
                parentHeight: window.innerHeight,
            };
        }
        else {
            return;
        }
    };
    RawPopover.prototype.handleResize = function () {
        this.updatePosition();
    };
    RawPopover.prototype.handleScroll = function () {
        var _a = this.props, onRequestClose = _a.onRequestClose, isOpened = _a.isOpened, disableCloseOnScroll = _a.disableCloseOnScroll;
        if (onRequestClose && isOpened && !disableCloseOnScroll) {
            onRequestClose();
        }
    };
    RawPopover = tslib_1.__decorate([
        pure_1.PURE,
        tslib_1.__metadata("design:paramtypes", [Object])
    ], RawPopover);
    return RawPopover;
}(React.Component));
exports.Popover = withTheme_1.withTheme(exports.POPOVER)(with_defaults_1.withDefaults({
    align: PopoverAlign.Left,
    placement: PopoverPlacement.Bottom,
})(RawPopover));
function stopPropagation(e) {
    e.stopPropagation();
}
function getArrowStyle(placement, align, offset) {
    switch (placement) {
        case PopoverPlacement.Top: //fallthrough
        case PopoverPlacement.Bottom: {
            switch (align) {
                case PopoverAlign.Left: {
                    return prefix_1.default({
                        left: offset,
                        transform: 'translateX(-50%)',
                    });
                }
                case PopoverAlign.Center: {
                    return prefix_1.default({
                        left: '50%',
                        transform: 'translateX(-50%)',
                    });
                }
                case PopoverAlign.Right: {
                    return prefix_1.default({
                        right: offset,
                        transform: 'translateX(50%)',
                    });
                }
            }
            break;
        }
        case PopoverPlacement.Left: //fallthrough
        case PopoverPlacement.Right: {
            switch (align) {
                case PopoverAlign.Top: {
                    return prefix_1.default({
                        top: offset,
                        transform: 'translateY(-50%)',
                    });
                }
                case PopoverAlign.Middle: {
                    return prefix_1.default({
                        top: '50%',
                        transform: 'translateY(-50%)',
                    });
                }
                case PopoverAlign.Bottom: {
                    return prefix_1.default({
                        bottom: offset,
                        transform: 'translateY(50%)',
                    });
                }
            }
        }
    }
    return undefined;
}
function movePopoverVertically(placement, align, anchorTop, anchorBottom, popoverHeight, parentHeight, checkBounds) {
    if (checkBounds === void 0) { checkBounds = false; }
    switch (placement) {
        case PopoverPlacement.Top: {
            var top_1 = anchorTop - popoverHeight;
            if (checkBounds && top_1 < 0) {
                return movePopoverVertically(PopoverPlacement.Bottom, align, anchorTop, anchorBottom, popoverHeight, parentHeight);
            }
            return {
                top: top_1,
                placement: placement,
                align: align,
            };
        }
        case PopoverPlacement.Bottom: {
            var top_2 = anchorBottom;
            if (checkBounds && top_2 + popoverHeight > parentHeight) {
                return movePopoverVertically(PopoverPlacement.Top, align, anchorTop, anchorBottom, popoverHeight, parentHeight);
            }
            return {
                top: top_2,
                placement: placement,
                align: align,
            };
        }
    }
    switch (align) {
        case PopoverAlign.Top: {
            var top_3 = anchorTop;
            if (checkBounds && top_3 + popoverHeight > parentHeight) {
                var resultForMiddle = movePopoverVertically(placement, PopoverAlign.Middle, anchorTop, anchorBottom, popoverHeight, parentHeight);
                if (resultForMiddle && resultForMiddle.top + popoverHeight > parentHeight) {
                    return movePopoverVertically(placement, PopoverAlign.Bottom, anchorTop, anchorBottom, popoverHeight, parentHeight);
                }
                return resultForMiddle;
            }
            return {
                top: top_3,
                placement: placement,
                align: align,
            };
        }
        case PopoverAlign.Middle: {
            var top_4 = anchorTop + (anchorBottom - anchorTop) / 2 - popoverHeight / 2;
            if (checkBounds) {
                if (top_4 < 0) {
                    return movePopoverVertically(placement, PopoverAlign.Top, anchorTop, anchorBottom, popoverHeight, parentHeight);
                }
                else if (top_4 + popoverHeight > parentHeight) {
                    return movePopoverVertically(placement, PopoverAlign.Bottom, anchorTop, anchorBottom, popoverHeight, parentHeight);
                }
            }
            return {
                top: top_4,
                placement: placement,
                align: align,
            };
        }
        case PopoverAlign.Bottom: {
            var top_5 = anchorBottom - popoverHeight;
            if (checkBounds && top_5 < 0) {
                var resultForMiddle = movePopoverVertically(placement, PopoverAlign.Middle, anchorTop, anchorBottom, popoverHeight, parentHeight);
                if (resultForMiddle && resultForMiddle.top < 0) {
                    return movePopoverVertically(placement, PopoverAlign.Top, anchorTop, anchorBottom, popoverHeight, parentHeight);
                }
                return resultForMiddle;
            }
            return {
                top: top_5,
                placement: placement,
                align: align,
            };
        }
    }
    return undefined;
}
function movePopoverHorizontally(placement, align, anchorLeft, anchorRight, popoverWidth, parentWidth, checkBounds) {
    if (checkBounds === void 0) { checkBounds = false; }
    switch (placement) {
        case PopoverPlacement.Left: {
            var left = anchorLeft - popoverWidth;
            if (checkBounds && left < 0) {
                return movePopoverHorizontally(PopoverPlacement.Right, align, anchorLeft, anchorRight, popoverWidth, parentWidth);
            }
            return {
                left: left,
                placement: placement,
                align: align,
            };
        }
        case PopoverPlacement.Right: {
            var left = anchorRight;
            if (checkBounds && left + popoverWidth > parentWidth) {
                return movePopoverHorizontally(PopoverPlacement.Left, align, anchorLeft, anchorRight, popoverWidth, parentWidth);
            }
            return {
                left: left,
                placement: placement,
                align: align,
            };
        }
    }
    switch (align) {
        case PopoverAlign.Left: {
            var left = anchorLeft;
            if (checkBounds && left + popoverWidth > parentWidth) {
                var resultForCenter = movePopoverHorizontally(placement, PopoverAlign.Center, anchorLeft, anchorRight, popoverWidth, parentWidth);
                if (resultForCenter && resultForCenter.left + popoverWidth > parentWidth) {
                    return movePopoverHorizontally(placement, PopoverAlign.Right, anchorLeft, anchorRight, popoverWidth, parentWidth);
                }
                return resultForCenter;
            }
            return {
                left: left,
                placement: placement,
                align: align,
            };
        }
        case PopoverAlign.Center: {
            var left = anchorLeft + (anchorRight - anchorLeft) / 2 - popoverWidth / 2;
            if (checkBounds) {
                if (left < 0) {
                    return movePopoverHorizontally(placement, PopoverAlign.Left, anchorLeft, anchorRight, popoverWidth, parentWidth);
                }
                else if (left + popoverWidth > parentWidth) {
                    return movePopoverHorizontally(placement, PopoverAlign.Right, anchorLeft, anchorRight, popoverWidth, parentWidth);
                }
            }
            return {
                left: left,
                placement: placement,
                align: align,
            };
        }
        case PopoverAlign.Right: {
            var left = anchorRight - popoverWidth;
            if (checkBounds && left < 0) {
                var resultForCenter = movePopoverHorizontally(placement, PopoverAlign.Center, anchorLeft, anchorRight, popoverWidth, parentWidth);
                if (resultForCenter && resultForCenter.left < 0) {
                    return movePopoverHorizontally(placement, PopoverAlign.Left, anchorLeft, anchorRight, popoverWidth, parentWidth);
                }
                return resultForCenter;
            }
            return {
                left: left,
                placement: placement,
                align: align,
            };
        }
    }
    return undefined;
}
function getPlacementModifier(placement) {
    switch (placement) {
        case PopoverPlacement.Bottom: {
            return 'placementBottom';
        }
        case PopoverPlacement.Top: {
            return 'placementTop';
        }
        case PopoverPlacement.Left: {
            return 'placementLeft';
        }
        case PopoverPlacement.Right: {
            return 'placementRight';
        }
    }
}
//# sourceMappingURL=Popover.js.map